@@ -4,14 +4,16 @@
 .knn <- function(sim, k) apply(sim, MARGIN=1, FUN=function(x) head(
 		    order(x, decreasing=TRUE, na.last=TRUE), k))
 
+.BIN_UBCF_param <- list( 
+  method = "jaccard", 
+  nn = 25, 
+  weighted = TRUE,
+  sample = FALSE
+)
+
 BIN_UBCF <- function(data, parameter = NULL){
 
-    p <- .get_parameters(list( 
-                    method = "jaccard", 
-                    nn = 25, 
-                    weighted = TRUE,
-		    sample = FALSE
-                    ), parameter) 
+    p <- .get_parameters(.BIN_UBCF_param, parameter) 
 
     if(p$sample) data <- sample(data, p$sample)
 
@@ -20,7 +22,15 @@ BIN_UBCF <- function(data, parameter = NULL){
 	data = data
 	), p )
 
-    predict <- function(model, newdata, n=10, ...) {
+    predict <- function(model, newdata, n=10, data=NULL, ...) {
+
+	## newdata are userid
+	if(is.numeric(newdata)) {
+	    if(is.null(data) || !is(data, "ratingMatrix"))
+		stop("If newdata is a user id then data needes to be the training dataset.")
+	    newdata <- data[newdata,]
+	}
+
 
         ## prediction
         ## FIXME: add Weiss dissimilarity
@@ -35,7 +45,7 @@ BIN_UBCF <- function(data, parameter = NULL){
 	    s_uk <- sapply(1:nrow(sim), FUN=function(x) 
 		    sim[x, neighbors[,x]])
 
-	    sum_s_uk <- colSums(s_uk)
+	    sum_s_uk <- colSums(s_uk, na.rm=TRUE)
 
 	    ## calculate the weighted sum
 	    r_a_norms <- sapply(1:nrow(newdata), FUN=function(i) {
@@ -64,16 +74,19 @@ BIN_UBCF <- function(data, parameter = NULL){
             ntrain = nrow(data), model = model, predict = predict)
 }
 
+.REAL_UBCF_param <- list( 
+  method = "cosine", 
+  nn = 25, 
+  sample = FALSE,
+  ## FIXME: implement weighted = TRUE,
+  normalize="center",
+  minRating = NA
+)
+
+
 REAL_UBCF <- function(data, parameter = NULL){
 
-    p <- .get_parameters(list( 
-                    method = "cosine", 
-                    nn = 25, 
-                    sample = FALSE,
-		    ## FIXME: implement weighted = TRUE,
-		    normalize="center",
-		    minRating = NA
-                    ), parameter) 
+    p <- .get_parameters(.REAL_UBCF_param, parameter) 
 
     if(p$sample) data <- sample(data, p$sample)
     
@@ -86,10 +99,17 @@ REAL_UBCF <- function(data, parameter = NULL){
 	), p)
 
     predict <- function(model, newdata, n=10, 
-	    type=c("topNList", "ratings"), ...) {
+	    data=NULL, type=c("topNList", "ratings"), ...) {
 
 	type <- match.arg(type)
 	
+	## newdata are userid
+	if(is.numeric(newdata)) {
+	    if(is.null(data) || !is(data, "ratingMatrix"))
+		stop("If newdata is a user id then data needes to be the training dataset.")
+	    newdata <- data[newdata,]
+	}
+	
 	if(!is.null(model$normalize)) 
 	    newdata <- normalize(newdata, method=model$normalize)
 
@@ -139,11 +159,13 @@ REAL_UBCF <- function(data, parameter = NULL){
 ## register recommender
 recommenderRegistry$set_entry(
         method="UBCF", dataType = "binaryRatingMatrix", fun=BIN_UBCF, 
-        description="Recommender based on user-based collaborative filtering (binary data).")
+        description="Recommender based on user-based collaborative filtering (binary data).",
+        parameters=.BIN_UBCF_param)
 
 
 recommenderRegistry$set_entry(
 	method="UBCF", dataType = "realRatingMatrix", fun=REAL_UBCF,
-	description="Recommender based on user-based collaborative filtering (real data).")
+	description="Recommender based on user-based collaborative filtering (real data).",
+  parameters=.REAL_UBCF_param)
 
 
